@@ -421,6 +421,10 @@ char *browse(char *path)
 		dir = NULL;
 	}
 
+#ifndef NANO_TINY
+	resized_for_browser = FALSE;
+#endif
+
 	/* If something was selected before, reselect it;
 	 * otherwise, just select the first item (..). */
 	if (present_name != NULL) {
@@ -486,11 +490,12 @@ char *browse(char *path)
 
 		function = interpret(kbinput);
 
-		if (function == do_help || function == full_refresh) {
-			function();
-#ifndef NANO_TINY
-			/* Simulate a terminal resize to force a directory reread,
-			 * or because the terminal dimensions might have changed. */
+		if (function == do_help)
+			do_help();
+		else if (function == full_refresh) {
+#ifdef NANO_TINY
+			full_refresh();
+#else
 			kbinput = THE_WINDOW_RESIZED;
 		} else if (function == do_toggle && get_shortcut(kbinput)->toggle == NO_HELP) {
 			TOGGLE(NO_HELP);
@@ -556,7 +561,7 @@ char *browse(char *path)
 							/* TRANSLATORS: This is a prompt. */
 							browser_refresh, _("Go To Directory")) < 0) {
 				statusbar(_("Cancelled"));
-				continue;
+				goto testresize;
 			}
 
 			path = free_and_assign(path, real_dir_from_tilde(answer));
@@ -573,7 +578,7 @@ char *browse(char *path)
 				 * the option --operatingdir, not of --restricted. */
 				statusline(ALERT, _("Can't go outside of %s"), operating_dir);
 				path = mallocstrcpy(path, present_path);
-				continue;
+				goto testresize;
 			}
 #endif
 			/* Snip any trailing slashes, so the name can be compared. */
@@ -644,9 +649,10 @@ char *browse(char *path)
 		} else
 			unbound_key(kbinput);
 
+  testresize:
 #ifndef NANO_TINY
 		/* If the terminal resized (or might have), refresh the file list. */
-		if (kbinput == THE_WINDOW_RESIZED) {
+		if (kbinput == THE_WINDOW_RESIZED || resized_for_browser) {
 			/* Remember the selected file, to be able to reselect it. */
 			present_name = copy_of(filelist[selected]);
 			goto read_directory_contents;
