@@ -1,7 +1,7 @@
 /**************************************************************************
  *   move.c  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2024 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2025 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014-2018 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -15,7 +15,7 @@
  *   See the GNU General Public License for more details.                 *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
- *   along with this program.  If not, see http://www.gnu.org/licenses/.  *
+ *   along with this program.  If not, see https://gnu.org/licenses/.     *
  *                                                                        *
  **************************************************************************/
 
@@ -189,7 +189,7 @@ void to_top_row(void)
 
 	set_proper_index_and_pww(&leftedge, offset, FALSE);
 
-	refresh_needed = (openfile->mark);
+	refresh_needed = (openfile->mark != NULL);
 }
 
 /* Place the cursor on the last row in the viewport, when possible. */
@@ -205,7 +205,7 @@ void to_bottom_row(void)
 	go_forward_chunks(editwinrows - 1, &openfile->current, &leftedge);
 	set_proper_index_and_pww(&leftedge, offset, TRUE);
 
-	refresh_needed = (openfile->mark);
+	refresh_needed = (openfile->mark != NULL);
 }
 
 /* Put the cursor line at the center, then the top, then the bottom. */
@@ -227,7 +227,9 @@ void do_cycle(void)
 /* Scroll the line with the cursor to the center of the screen. */
 void do_center(void)
 {
-	do_cycle();  /* The main loop has set 'cycling_aim' to zero. */
+	adjust_viewport(CENTERING);
+	draw_all_subwindows();
+	full_refresh();
 }
 #endif /* !NANO_TINY */
 
